/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp org.ukui.mpris.controller.xml -a mpris_controller_adaptor -c MprisControllerAdaptor
 *
 * qdbusxml2cpp is Copyright (C) 2020 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "mpris_controller_adaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class MprisControllerAdaptor
 */

MprisControllerAdaptor::MprisControllerAdaptor(QObject *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

MprisControllerAdaptor::~MprisControllerAdaptor()
{
    // destructor
}

QStringList MprisControllerAdaptor::supportOperations() const
{
    // get the value of property supportOperations
    return qvariant_cast< QStringList >(parent()->property("supportOperations"));
}

void MprisControllerAdaptor::operation(const QString &operation)
{
    // handle method call org.ukui.mpris.controller.operation
    QMetaObject::invokeMethod(parent(), "operation", Q_ARG(QString, operation));
}

