/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp usd-sharing-manager.xml -a sharing-adaptor
 *
 * qdbusxml2cpp is Copyright (C) 2020 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "sharing-adaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class SharingAdaptor
 */

SharingAdaptor::SharingAdaptor(QObject *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

SharingAdaptor::~SharingAdaptor()
{
    // destructor
}

void SharingAdaptor::DisableService(const QString &serviceName)
{
    // handle method call org.ukui.SettingsDaemon.Sharing.DisableService
    QMetaObject::invokeMethod(parent(), "DisableService", Q_ARG(QString, serviceName));
}

void SharingAdaptor::EnableService(const QString &serviceName)
{
    // handle method call org.ukui.SettingsDaemon.Sharing.EnableService
    QMetaObject::invokeMethod(parent(), "EnableService", Q_ARG(QString, serviceName));
}

