/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp ukui-settings-daemon.xml -i plugin-manager.h -p manager-interface
 *
 * qdbusxml2cpp is Copyright (C) 2019 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "manager-interface.h"
#include "clib-syslog.h"
/*
 * Implementation of interface class OrgUkuiSettingsDaemonInterface
 */

PluginManagerDBus::PluginManagerDBus(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent)
    : QDBusAbstractInterface(service, path, staticInterfaceName(), connection, parent)
{
}

PluginManagerDBus::~PluginManagerDBus()
{
//    USD_LOG(LOG_DEBUG,"exit....");
}

