/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp org.mpris.MediaPlayer2.Player.xml -p org_mpris_MediaPlayer2_Player
 *
 * qdbusxml2cpp is Copyright (C) 2020 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "org_mpris_MediaPlayer2_Player.h"

/*
 * Implementation of interface class OrgMprisMediaPlayer2PlayerInterface
 */

OrgMprisMediaPlayer2PlayerInterface::OrgMprisMediaPlayer2PlayerInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent)
    : QDBusAbstractInterface(service, path, staticInterfaceName(), connection, parent)
{
}

OrgMprisMediaPlayer2PlayerInterface::~OrgMprisMediaPlayer2PlayerInterface()
{
}

