/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp usd-xrandr-manager.xml -a xrandr-adaptor
 *
 * qdbusxml2cpp is Copyright (C) 2020 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "xrandr-adaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include "clib-syslog.h"
/*
 * Implementation of adaptor class WaylandAdaptor
 */

XrandrAdaptor::XrandrAdaptor(QObject *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

XrandrAdaptor::~XrandrAdaptor()
{
    // destructor
}

int XrandrAdaptor::setScreensParam(const QString &screensParam, const QString &appName){
    int out0;
    USD_LOG(LOG_DEBUG," appName:%s", appName.toLatin1().data());
    QMetaObject::invokeMethod(parent(), "setScreensParam", Q_RETURN_ARG(int, out0),  Q_ARG(QString, screensParam),  Q_ARG(QString, appName));
    return out0;
}

QString XrandrAdaptor::getScreensParam( const QString &appName){
    QString out0;
    USD_LOG(LOG_DEBUG," appName:%s", appName.toLatin1().data());
    QMetaObject::invokeMethod(parent(), "getScreensParam", Q_RETURN_ARG(QString, out0),  Q_ARG(QString, appName));
    return out0;
}

int XrandrAdaptor::setScreenMode(const QString &modeName, const QString &appName){
    int out0;
    USD_LOG(LOG_DEBUG,"change screen :%s, appName:%s",modeName.toLatin1().data(), appName.toLatin1().data());
    QMetaObject::invokeMethod(parent(), "setScreenMode", Q_RETURN_ARG(int, out0),  Q_ARG(QString, modeName),  Q_ARG(QString, appName));
    return out0;
}

int XrandrAdaptor::getScreenMode(const QString &appName){
    int out0;
    USD_LOG(LOG_DEBUG,"get screen mode request from appName:%s", appName.toLatin1().data());
    QMetaObject::invokeMethod(parent(), "getScreenMode", Q_RETURN_ARG(int, out0),  Q_ARG(QString, appName));
    return out0;
}

