/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp usd-sharing-manager.xml -a sharing-adaptor
 *
 * qdbusxml2cpp is Copyright (C) 2020 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef SHARING-ADAPTOR_H
#define SHARING-ADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
QT_BEGIN_NAMESPACE
class QByteArray;
template<class T> class QList;
template<class Key, class Value> class QMap;
class QString;
class QStringList;
class QVariant;
QT_END_NAMESPACE

/*
 * Adaptor class for interface org.ukui.SettingsDaemon.Sharing
 */
class SharingAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.ukui.SettingsDaemon.Sharing")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.ukui.SettingsDaemon.Sharing\">\n"
"    <signal name=\"serviceChange\">\n"
"      <arg direction=\"out\" type=\"s\" name=\"state\"/>\n"
"      <arg direction=\"out\" type=\"s\" name=\"serviceName\"/>\n"
"    </signal>\n"
"    <method name=\"EnableService\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"serviceName\"/>\n"
"    </method>\n"
"    <method name=\"DisableService\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"serviceName\"/>\n"
"    </method>\n"
"  </interface>\n"
        "")
public:
    SharingAdaptor(QObject *parent);
    virtual ~SharingAdaptor();

public: // PROPERTIES
public Q_SLOTS: // METHODS
    void DisableService(const QString &serviceName);
    void EnableService(const QString &serviceName);
Q_SIGNALS: // SIGNALS
    void serviceChange(const QString &state, const QString &serviceName);
};

#endif
